#!/bin/bash

SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE}")" >/dev/null; pwd)" || exit 1

MC=(/usr/bin/mc '--insecure')
[[ -z "${MC_CONFIG_DIR}" ]] || {
  MC+=(--config-dir "${MC_CONFIG_DIR}")
}

# connectToMinio
# Use a check-sleep-check loop to wait for Minio service to be available
connectToMinio() {
  local attempts=0
  local limit=30 # Allow 30 attempts

  local access_key=$(cat /config/rootUser) &&
  local secret_key=$(cat /config/rootPassword) || {
    echo "Unable to read credentials"
    return 1
  } >&2

  local url="${MINIO_SCHEME}://${MINIO_ENDPOINT}:${MINIO_PORT}"
  echo "Connecting to Minio server: ${url}"
  local command=("${MC[@]}" config host add myminio "${url}" "${access_key}" "${secret_key}")
  until "${command[@]}"
  do
    attempts=$((attempts + 1))
    echo \"Failed attempts: $attempts\"
    [[ $attempts < $limit ]] || exit 1
    sleep 10 # 10 second intervals between attempts
  done
  return 0
}


# checkUserExists $username
# Check if the user exists, by using the exit code of `mc admin user info`
checkUserExists() {
  local user="$1"; shift
  "${MC[@]}" admin user info myminio "${user}" > /dev/null 2>&1
}

# createUser ($usersecret,$policy)
createUser() {
  local user_line=$1; shift

  local user_secret_name="$(cut -d ',' -f 1 <<<"${user_line}")" || {
    echo "Unable to get user secret name"
    return 1
  } >&2
  local policy="$(cut -d ',' -f 2 <<<"${user_line}")" || {
    echo "Unable to get policy for user '${user_secret_name}'"
    return 1
  } >&2
  local user=$(cat /etc/minio/users/${user_secret_name}/accessKey) || {
    echo "Unable to get access key for user '${user_secret_name}'"
    return 1
  } >&2
  local pass=$(cat /etc/minio/users/${user_secret_name}/secretKey) || {
    echo "Unable to get secret key for user '${user_secret_name}'"
    return 1
  } >&2

  # Create the user if it does not exist
  if ! checkUserExists "${user}" ; then
    echo "Creating user '${user}'"
    "${MC[@]}" admin user add myminio "${user}" "${pass}" || {
      echo "Unable to create user '${user}'"
      return 1
    } >&2
  else
    echo "User '${user}' already exists."
  fi


  # set policy for user
  if [[ ! -z "${policy}" && "${policy}" != " " ]]; then
      echo "Adding policy '${policy}' for '${user}'"
      "${MC[@]}" admin policy set myminio "${policy}" "user=${user}" || {
        echo "Unable to attach policy '${policy}' to user '${user}'"
        return 1
      } >&2
  else
      echo "User '${user}' has no policy attached."
  fi
}

# Try connecting to Minio instance
connectToMinio || exit $?

readarray rows < "${SCRIPT_DIR}/user_list"
RC=0
for row in "${rows[@]}";do
  row_array=(${row})
  createUser "${row_array[0]}" || RC=$?
done

exit $RC
