#!/bin/bash

SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE}")" >/dev/null; pwd)" || exit 1

MC=(/usr/bin/mc '--insecure')
[[ -z "${MC_CONFIG_DIR}" ]] || {
  MC+=(--config-dir "${MC_CONFIG_DIR}")
}

# connectToMinio
# Use a check-sleep-check loop to wait for Minio service to be available
connectToMinio() {
  local attempts=0
  local limit=30 # Allow 30 attempts

  local access_key=$(cat /config/rootUser) &&
  local secret_key=$(cat /config/rootPassword) || {
    echo "Unable to read credentials"
    return 1
  } >&2

  local url="${MINIO_SCHEME}://${MINIO_ENDPOINT}:${MINIO_PORT}"
  echo "Connecting to Minio server: ${url}"
  local command=("${MC[@]}" config host add myminio "${url}" "${access_key}" "${secret_key}")
  until "${command[@]}"
  do
    attempts=$((attempts + 1))
    echo \"Failed attempts: $attempts\"
    [[ $attempts < $limit ]] || exit 1
    sleep 10 # 10 second intervals between attempts
  done
  return 0
}

configure() {
  local rc=0

  echo "set scanner delay"
  "${MC[@]}" admin config set myminio/ scanner delay=3000000.0 || rc=$?

  echo "set scanner max_wait"
  "${MC[@]}" admin config set myminio/ scanner max_wait=26300000s || rc=$?

  echo "set scanner cycle"
  "${MC[@]}" admin config set myminio/ scanner cycle=26300000s || rc=$?

  return $rc
}

# checkBucketExists ($bucket)
# Check if the bucket exists, by using the exit code of `mc ls`
checkBucketExists() {
  local bucket="$1"; shift
  "${MC[@]}" ls "myminio/${bucket}" > /dev/null 2>&1
}

# createBucket ($bucket, $policy, $purge)
# Ensure bucket exists, purging if asked to
createBucket() {
  local bucket_line="$1"; shift

  local bucket="$(cut -d ',' -f 1 <<<"${bucket_line}")" || {
    echo "Unable to get bucket name"
    return 1
  } >&2
  local policy="$(cut -d ',' -f 2 <<<"${bucket_line}")" || {
    echo "Unable to get policy for bucket '${bucket}'"
    return 1
  } >&2
  local purge="$(cut -d ',' -f 3 <<<"${bucket_line}")" || {
    echo "Unable to get purge flag for bucket '${bucket}'"
    return 1
  } >&2
  local versioning="$(cut -d ',' -f 4 <<<"${bucket_line}")" || {
    echo "Unable to get versioning for bucket '${bucket}'"
    return 1
  } >&2

  # Purge the bucket, if set & exists
  # Since PURGE is user input, check explicitly for `true`
  if [[ "${purge}" == 'true' ]]; then
    if checkBucketExists "${bucket}" ; then
      echo "Purging bucket '${bucket}'."
      "${MC[@]}" rm -r --force "myminio/${bucket}"
    else
      echo "Bucket '${bucket}' does not exist, skipping purge."
    fi
  fi

  # Create the bucket if it does not exist
  if ! checkBucketExists "${bucket}"; then
    echo "Creating bucket '${bucket}'"
    "${MC[@]}" mb "myminio/${bucket}" || {
      echo "Unable to create bucket"
      return 1
    } >&2
  else
    echo "Bucket '${bucket}' already exists."
  fi


  # set versioning for bucket
  if [[ ! -z "${versioning}" ]]; then
    local action=''
    case "${versioning}" in
      true)
        action='enable'
        ;;
      false)
        action='disable'
        ;;
      *)
        echo "Unsupported versioning value for bucket '${bucket}': '${versioning}'" >&2
        return 1
        ;;
    esac
    echo "${action} versioning for '${bucket}'"
    "${MC[@]}" version "${action}" "myminio/${bucket}" || {
      echo "Unable to ${action} versioning for bucket '${bucket}'"
      return 1
    } >&2
  else
      echo "Bucket '${bucket}' versioning unchanged."
  fi

  # At this point, the bucket should exist, skip checking for existence
  # Set policy on the bucket
  echo "Setting policy of bucket '${bucket}' to '${policy}'."
  "${MC[@]}" policy set "${policy}" "myminio/${bucket}" || {
    echo "Unable to set policy '${policy}' for bucket '${bucket}'"
    return 1
  } >&2
}

# Try connecting to Minio instance
connectToMinio

# Configure minio
configure

readarray rows < "${SCRIPT_DIR}/bucket_list"
RC=0
for row in "${rows[@]}";do
  row_array=(${row})
  createBucket "${row_array[0]}" || RC=$?
done

exit $RC
