#!/bin/bash

realm=$REALM_NAME
customerUserLogin=$CUSTOMER_USER_LOGIN
customerPassword=$CUSTOMER_PASSWORD
keycloakSaagieUserPassword=$KEYCLOAK_SAAGIE_USER_PWD
keycloakUrl="http://$KEYCLOAK_HOST:$KEYCLOAK_PORT/auth"
keycloakAdminPassword=$KEYCLOAK_ADMIN_PWD
authUrl="http://$AUTH_HOST:$AUTH_PORT/api"
authenticationUrl="http://$AUTHENTICATION_HOST:$AUTHENTICATION_PORT/api"
m2mUsername=$M2M_USERNAME
m2mPassword=$M2M_PWD
superAdminGroup=$SUPER_ADMIN_GROUP
adminGroup=$ADMIN_GROUP
localDir=${LOCAL_DIR:-/shared}
provider=$PROVIDER

# LDAP PARAMS
userdn=$LDAP_USERDN
basedn=$LDAP_BASEDN
ldapHost=$LDAP_HOST
userObjectClasses=$LDAP_USER_OBJECT_CLASSES
usernameLDAPAttribute=$LDAP_USERNAME_LDAP_ATTRIBUTE
bindDnAttribute=$LDAP_BINDDN_ATTRIBUTE
rdnLDAPAttribute=$LDAP_RDN_LDAP_ATTRIBUTE
vendor=$LDAP_VENDOR
editMode=$LDAP_EDIT_MODE
uuidLDAPAttribute=$LDAP_UUID_LDAP_ATTRIBUTE
membershipAttributeType=$LDAP_GROUP_MEMBERSHIP_ATTRIBUTE_TYPE
groupNameLdapAttribute=$LDAP_GROUP_NAME_LDAP_ATTRIBUTE
groupdn=$LDAP_GROUP_GROUPDN
membershipLdapAttribute=$LDAP_GROUP_MEMBERSHIP_LDAP_ATTRIBUTE
groupObjectClasses=$LDAP_GROUP_OBJECT_CLASSES

# SSO PARAMS
ssoAlias=$SSO_ALIAS
ssoServiceUrl=$SSO_SERVICE_URL
ssoTokenUrl="$SSO_SERVICE_URL/t/$SSO_ALIAS/oauth/token"
ssoUserInfoUrl="$SSO_SERVICE_URL/t/$SSO_ALIAS/userinfo"
ssoAuthorizationUrl="$SSO_SERVICE_URL/t/$SSO_ALIAS/authorize"
ssoClientId=$SSO_EXCHANGE_CLIENT_ID
ssoClientSecret=$SSO_EXCHANGE_CLIENT_SECRET

function keycloak() {
    baseUrl=$keycloakUrl
    authHeader=$keycloakAuthHeader
    realmHeader=""
    "$@"
}

function auth() {
    baseUrl=$authUrl
    authHeader=$authAuthHeader
    realmHeader="Saagie-Realm: $realm"
    "$@"
}

function post() {
	url=$1
	data=$2
	curl -s --fail --request POST \
  		--url $baseUrl$url \
  		--header "$authHeader" \
  		--header "$realmHeader" \
  		--header 'content-type: application/json' \
  		--data "$data"
}

function get() {
	url=$1
	curl -s --fail --request GET \
  		--url $baseUrl$url \
  		--header "$realmHeader" \
  		--header "$authHeader"
}

function delete() {
	url=$1
	curl -s --request DELETE \
  		--url $baseUrl$url \
  		--header "$realmHeader" \
  		--header "$authHeader"
}

function put() {
	url=$1
	data=$2
	curl -s --fail --request PUT \
	  --url $baseUrl$url \
	  --header "$authHeader" \
	  --header "$realmHeader" \
	  --header 'content-type: application/json' \
	  --data "$data"

}

function escaped() {
	cat $1 | sed -e 's/\\/\\\\/g' | sed -E ':a;N;$!ba;s/\n/\\n/g'  | sed -e 's/\"/\\\"/g'
}

function getBackendClientId() {
    echo -n 'Getting backend client ID : '
    backendClientId=`keycloak get "/admin/realms/$realm/clients" | jq -r '.[] | select(.clientId=="backend") | .id'` || {
      echo "Unable to get backend client id"
      return 1
    } >&2
    echo $backendClientId

    echo -n 'Retrieving backend client secret : '
    clientSecret=`keycloak get "/admin/realms/$realm/clients/$backendClientId/client-secret" | jq -r '.value'` || {
      echo "Unable to get client secret"
      return 1
    } >&2
    echo $clientSecret
    echo $clientSecret > "$localDir/clientsecret"
}

function createLdapConfig() {
    echo 'Adding ldap configuration'
    keycloak post "/admin/realms/$realm/components" '{
          "name": "ldap",
          "providerId": "ldap",
          "providerType": "org.keycloak.storage.UserStorageProvider",
          "config": {
            "fullSyncPeriod": [
              "600"
            ],
            "usersDn": [
              "'"$userdn"','"$basedn"'"
            ],
            "userObjectClasses": [
              "'"$userObjectClasses"'"
            ],
            "usernameLDAPAttribute": [
              "'"$usernameLDAPAttribute"'"
            ],
            "bindDn": [
              "'"$bindDnAttribute"'='"$m2mUsername"','"$userdn"','"$basedn"'"
            ],
            "bindCredential": [
              "'"$m2mPassword"'"
            ],
            "rdnLDAPAttribute": [
              "'"$rdnLDAPAttribute"'"
            ],
            "vendor": [
              "'"$vendor"'"
            ],
            "editMode": [
              "'"$editMode"'"
            ],
            "uuidLDAPAttribute": [
              "'"$uuidLDAPAttribute"'"
            ],
            "connectionUrl": [
              "ldap://'"$ldapHost"'"
            ],
            "syncRegistrations": [
              "false"
            ]
          }
        }'
}

function updateExecutionFlow() {
    local rc=0
    echo "Getting flow execution ids"

    executionIds=`keycloak get "/admin/realms/$realm/authentication/flows/saagie_direct_grant/executions" | jq -r '[ .[] | select(.providerId=="auth-script-based") | .id ]'` || {
      echo "Unable to get execution ids"
      return 1
    } >&2

    expirationScriptExecutionId=`echo $executionIds | jq -r '.[0]'` &&
    userLockedScriptExecutionId=`echo $executionIds | jq -r '.[1]'`;
    rc=$?
    echo "Expiration script execution id : $expirationScriptExecutionId"
    echo "User-locked script execution id : $userLockedScriptExecutionId"

    [[ $rc == 0 ]] || return $rc

    echo "Updating expiration script execution config"

    keycloak post "/admin/realms/$realm/authentication/executions/$expirationScriptExecutionId/config" '{
    "config": {
     "scriptName": "expirationScript",
     "scriptDescription": "failed if password expired",
     "scriptCode": "'"$(escaped expiration-script-$provider.javascript)"'"
    },
    "alias": "expirationScript"
    }' || return $?

    echo 'Updating expiration script execution'
    keycloak put "/admin/realms/$realm/authentication/flows/saagie_direct_grant/executions" '{
      "id" : "'"$expirationScriptExecutionId"'",
      "requirement" : "REQUIRED",
      "displayName" : "Script",
      "alias" : "expirationScript",
      "requirementChoices" : [ "REQUIRED", "OPTIONAL", "DISABLED" ],
      "providerId" : "auth-script-based",
      "configurable" : true,
      "level" : 0,
      "index" : 4
    }' || return $?

    echo "Updating userlocked script execution config"
    keycloak post "/admin/realms/$realm/authentication/executions/$userLockedScriptExecutionId/config" '{
      "config": {
        "scriptName": "userLockedScript",
        "scriptDescription": "failed if user is locked",
        "scriptCode": "'"$(escaped user-locked-script.javascript)"'"
      },
      "alias": "userLockedScript"
    }' || return $?

    echo 'Updating userlocked script execution'
    keycloak put "/admin/realms/$realm/authentication/flows/saagie_direct_grant/executions" '{
       "id" : "'"$userLockedScriptExecutionId"'",
       "requirement" : "REQUIRED",
       "displayName" : "Script",
       "alias" : "userLockedScript",
       "requirementChoices" : [ "REQUIRED", "OPTIONAL", "DISABLED" ],
       "configurable" : true,
       "providerId" : "auth-script-based",
       "level" : 0,
       "index" : 2
    }' || return $?

    echo 'Raising userlocked script priority (3 times)'
    keycloak post "/admin/realms/$realm/authentication/executions/$userLockedScriptExecutionId/raise-priority" &&
    keycloak post "/admin/realms/$realm/authentication/executions/$userLockedScriptExecutionId/raise-priority" &&
    keycloak post "/admin/realms/$realm/authentication/executions/$userLockedScriptExecutionId/raise-priority"
}

function createGroupLdapConfig() {
    echo 'Adding groupldap attribute mapper'
    keycloak post "/admin/realms/$realm/components" '{
      "name": "group-ldap-mapper",
      "providerId": "group-ldap-mapper",
      "providerType": "org.keycloak.storage.ldap.mappers.LDAPStorageMapper",
      "parentId": "'"$ldapconfigId"'",
      "config": {
        "membership.attribute.type": [
          "'"$membershipAttributeType"'"
        ],
        "group.name.ldap.attribute": [
          "'"$groupNameLdapAttribute"'"
         ],
        "preserve.group.inheritance": [
          "true"
        ],
        "groups.dn": [
          "'"$groupdn"','"$basedn"'"
        ],
        "mapped.group.attributes": [],
        "mode": [
          "LDAP_ONLY"
        ],
        "user.roles.retrieve.strategy": [
          "LOAD_GROUPS_BY_MEMBER_ATTRIBUTE"
        ],
        "membership.ldap.attribute": [
          "'"$membershipLdapAttribute"'"
        ],
        "group.object.classes": [
          "'"$groupObjectClasses"'"
        ],
        "drop.non.existing.groups.during.sync": [
          "true"
        ]
      }
    }'
}

function createExpirationMapper() {
    echo 'Adding expiration attribute mapper'
    keycloak post "/admin/realms/$realm/components" '{
      "name": "expiration-attribute-ldap-mapper",
      "providerId": "user-attribute-ldap-mapper",
      "providerType": "org.keycloak.storage.ldap.mappers.LDAPStorageMapper",
      "parentId": "'"$ldapconfigId"'",
      "config": {
        "ldap.attribute": [
          "krbPasswordExpiration"
        ],
        "is.mandatory.in.ldap": [
          "false"
        ],
        "read.only": [
          "false"
        ],
        "always.read.value.from.ldap": [
          "true"
        ],
        "user.model.attribute": [
          "freeipaExpiration"
        ]
      }
    }'
}

function createFailureCounterMapper() {
    echo 'Adding failureCounter attribute mapper'
    keycloak post "/admin/realms/$realm/components" '{
      "name": "failureCounter-attribute-ldap-mapper",
      "providerId": "user-attribute-ldap-mapper",
      "providerType": "org.keycloak.storage.ldap.mappers.LDAPStorageMapper",
      "parentId": "'"$ldapconfigId"'",
      "config": {
        "ldap.attribute": [
          "krbLoginFailedCount"
        ],
        "is.mandatory.in.ldap": [
          "false"
        ],
        "read.only": [
          "false"
        ],
        "always.read.value.from.ldap": [
          "true"
        ],
        "user.model.attribute": [
          "freeipaFailureCounter"
        ]
      }
     }'
}

function createLastFailedAuthMapper() {
    echo 'Adding last failed auth attribute mapper'
    keycloak post "/admin/realms/$realm/components" '{
      "name": "lastFailedAuth-attribute-ldap-mapper",
      "providerId": "user-attribute-ldap-mapper",
      "providerType": "org.keycloak.storage.ldap.mappers.LDAPStorageMapper",
      "parentId": "'"$ldapconfigId"'",
      "config": {
        "ldap.attribute": [
          "krbLastFailedAuth"
        ],
        "is.mandatory.in.ldap": [
          "false"
        ],
        "read.only": [
          "true"
        ],
        "always.read.value.from.ldap": [
          "true"
        ],
        "user.model.attribute": [
          "lastFailedAuth"
        ]
      }
    }'
}

function updateFirstNameAttributeMapper() {
    local rc=0

    echo 'Updating firstName attribute mapper'
    echo -n 'Getting ldap configuration id: '
    ldapconfigId=`keycloak get "/admin/realms/$realm/components" | jq -r '.[] | select(.name=="ldap") | .id' `; rc=$?
    echo "Ldap configuration id is: $ldapconfigId"
    [[ $rc == 0 ]] || return $rc

    echo 'Getting ldap firstName attribute mapper id: '
    firstNameMapperId=`keycloak get "/admin/realms/$realm/components" | jq -r '.[] | select(.name=="first name") | .id' `; rc=$?
    echo "firstName attribute mapper id is: $firstNameMapperId"
    [[ $rc == 0 ]] || return $rc

    keycloak put "/admin/realms/$realm/components/$firstNameMapperId" '{
      "id": "'"$firstNameMapperId"'",
      "name": "first name",
      "providerId": "user-attribute-ldap-mapper",
      "providerType": "org.keycloak.storage.ldap.mappers.LDAPStorageMapper",
      "parentId": "'"$ldapconfigId"'",
      "config": {
        "ldap.attribute": [
          "givenName"
        ],
        "is.mandatory.in.ldap": [
          "true"
        ],
        "is.binary.attribute": [
          "false"
        ],
        "read.only": [
          "false"
        ],
        "always.read.value.from.ldap": [
          "true"
        ],
        "user.model.attribute": [
          "firstName"
        ]
      }
    }' || {
      echo "firstName attribute mapper failed"
      return 1
    } >&2
    echo 'firstName attribute mapper updated.'
}

function createFullNameAttributeMapper() {
    echo 'Adding fullname attribute mapper'
    keycloak post "/admin/realms/$realm/components" '{
      "name": "full-name-ldap-mapper",
      "providerId": "full-name-ldap-mapper",
      "providerType": "org.keycloak.storage.ldap.mappers.LDAPStorageMapper",
      "parentId": "'"$ldapconfigId"'",
      "config": {
        "read.only": [
          "false"
        ],
        "write.only": [
          "true"
        ],
        "ldap.full.name.attribute": [
           "cn"
         ]
       }
     }'
}

function createRoleLdapMapper() {
    echo 'Adding role-ldap attribute mapper'
    keycloak post "/admin/realms/$realm/components" '{
       "name" : "role-ldap-mapper",
       "providerId" : "role-ldap-mapper",
       "providerType": "org.keycloak.storage.ldap.mappers.LDAPStorageMapper",
       "parentId": "'"$ldapconfigId"'",
       "config" : {
         "mode" : [
           "LDAP_ONLY"
         ],
         "membership.attribute.type" : [
           "DN"
         ],
         "roles.dn" : [
           "cn=roles,cn=accounts,'"$basedn"'"
         ],
         "user.roles.retrieve.strategy" : [
           "LOAD_ROLES_BY_MEMBER_ATTRIBUTE"
         ],
         "membership.ldap.attribute" : [
           "member"
         ],
         "membership.user.ldap.attribute" : [
           "uid"
         ],
         "role.name.ldap.attribute" : [
           "cn"
         ],
         "use.realm.roles.mapping" : [
           "false"
         ],
         "role.object.classes" : [
           "groupOfNames"
         ],
         "client.id" : [
           "backend"
         ]
       }
     }'
}

function getUserGroups() {
    # We do not query ipa or keycloak directly because auth filters some groups
    # The groups we retrieve here will be inserted into security mongo database

    echo 'Retrieving user groups from ipa via auth '
    groups=`auth get "/groups"`
    echo "groups : $groups"

    echo $groups | jq -r '.[] | .name' > /shared/groups
}

function createRealm() {
    local rc=0

    echo "Creating $realm realm"
    keycloak post "/admin/realms" "{\"realm\": \"$realm\", \"ssoSessionIdleTimeout\": 43200, \"ssoSessionMaxLifespan\": 43200, \"offlineSessionIdleTimeout\": 43200, \"accessTokenLifespan\":43200, \"accessTokenLifespanForImplicitFlow\":43200, \"enabled\": true}" || return $?

    echo -n "Retrieving realm management id : "
    realmManagementId=`keycloak get "/admin/realms/$realm/clients" | jq -r '.[] | select(.clientId=="realm-management") | .id'`; rc=$?
    echo $realmManagementId
    [[ $rc == 0 ]] || return $rc

    echo "Updating realm management roles"
    keycloak put "/admin/realms/$realm/clients/$realmManagementId" '{"clientId": "realm-management","defaultRoles": ["manage-users","manage-clients"]}' || return $?

    echo "Creating user"
    keycloak post "/admin/realms/$realm/users" '{"username": "saagie-auth-keycloak","enabled": true}' || return $?

    echo -n "Retrieving user id : "
    saagieUserId=`keycloak get "/admin/realms/$realm/users" | jq -r '.[] | select(.username=="saagie-auth-keycloak") | .id'`; rc=0
    echo $saagieUserId
    [[ $rc == 0 ]] || return $rc

    echo "Setting user password"
    keycloak put "/admin/realms/$realm/users/$saagieUserId/reset-password" "{\"type\": \"password\",\"value\": \"$keycloakSaagieUserPassword\"}" || return $?

    echo -n "Getting manage-realm role id : "
    manageRealmRoleId=`keycloak get "/admin/realms/$realm/users/$saagieUserId/role-mappings/clients/$realmManagementId/available"  | jq -r '.[] | select(.name=="manage-realm") | .id'`; rc=$?
    echo $manageRealmRoleId
    [[ $rc == 0 ]] || return $rc

    echo "Adding managerealm role to saagie user"
    keycloak post "/admin/realms/$realm/users/$saagieUserId/role-mappings/clients/$realmManagementId" "[{\"id\":\"$manageRealmRoleId\",\"name\":\"manage-realm\"}]" || return $?

    echo "Copying direct grant flow"
    keycloak post "/admin/realms/$realm/authentication/flows/direct%20grant/copy" "{\"newName\": \"saagie_direct_grant\"}" || return $?

    echo "Creating flow execution"
    keycloak post "/admin/realms/$realm/authentication/flows/saagie_direct_grant/executions/execution" '{"provider":"auth-script-based" }' || return $?

    echo "Creating 2nd flow execution"
    keycloak post "/admin/realms/$realm/authentication/flows/saagie_direct_grant/executions/execution" '{"provider":"auth-script-based" }' || return $?

    updateExecutionFlow || return $?

    echo 'Update direct grant flow'
    keycloak put "/admin/realms/$realm" '{"directGrantFlow":"saagie_direct_grant"}' || return $?

    echo 'Creating frontend client'
    keycloak post "/admin/realms/$realm/clients" '{ "clientId": "frontend",
      "publicClient": true,
      "enabled": true,
      "directAccessGrantsEnabled": true
     }' || return $?

    echo 'Creating backend client'
    keycloak post "/admin/realms/$realm/clients" '{ "clientId": "backend",
      "publicClient": false,
      "enabled": true,
      "directAccessGrantsEnabled": true,
      "standardFlowEnabled": false
    }' || return $?

    getBackendClientId || return $?
    echo "provider $provider"

    if [[ "$provider" == "freeipa" ]] || [[ "$provider" == "ldap" ]]; then

      echo 'before createLdapConfig'
      createLdapConfig || return $?

      echo -n 'Getting ldap configuration id : '
      ldapconfigId=`keycloak get "/admin/realms/$realm/components" | jq -r '.[] | select(.name=="ldap") | .id' `; rc=$?
      echo $ldapconfigId
      [[ $rc == 0 ]] || return $rc

      createGroupLdapConfig || return $?

      if [[ "$provider" == "freeipa" ]]; then
          updateFirstNameAttributeMapper &&
          createExpirationMapper         &&
          createFailureCounterMapper     &&
          createLastFailedAuthMapper     &&
          createFullNameAttributeMapper  &&
          createRoleLdapMapper           ||
          return $?
      else
          echo "Provider is $provider. Ignoring mappers installation."
      fi

      echo 'Syncing ldap users/groups'
      keycloak post "/admin/realms/$realm/user-storage/$ldapconfigId/sync?action=triggerFullSync" || return $?

      if [[ "$provider" == "freeipa" ]]; then
        echo -ne '\nGetting roleldap attribute mapper id : '
        roleldapMapperId=`keycloak get "/admin/realms/$realm/components" | jq -r '.[] | select(.name=="role-ldap-mapper") | .id' `; rc=$?
        echo $roleldapMapperId
        [[ $rc == 0 ]] || return $rc

        echo 'Syncing ldap roles'
        keycloak post "/admin/realms/$realm/user-storage/$ldapconfigId/mappers/$roleldapMapperId/sync?direction=fedToKeycloak" || return $?
      else
          echo "Provider is $provider. Ignoring syncing role ldap attribute."
      fi
    else
      echo "Provider is $provider. Ignoring creating ldap config"
    fi

    if [[ "$provider" == "keycloak" ]] || [[ "$provider" == "sso" ]]; then
      echo "Creating keycloak super admin group"
      keycloak post "/admin/realms/$realm/groups" "{\"name\": \"$superAdminGroup\"}" || return $?

      echo "Creating keycloak admin group"
      keycloak post "/admin/realms/$realm/groups" "{\"name\": \"$adminGroup\"}" || return $?

      echo "Creating m2m user"
      keycloak post "/admin/realms/$realm/users" "{\"username\": \"$m2mUsername\",\"enabled\": true, \"groups\": [\"$superAdminGroup\"] }" || return $?

      echo -n "Retrieving m2m user id : "
      m2mUserId=`keycloak get "/admin/realms/$realm/users" | jq -r ".[] | select(.username==\"$m2mUsername\") | .id"`; rc=$?
      echo $m2mUserId
      [[ $rc == 0 ]] || return $rc

      echo "Setting m2m user password"
      keycloak put "/admin/realms/$realm/users/$m2mUserId/reset-password" "{\"type\": \"password\",\"value\": \"$m2mPassword\"}" || return $?

      echo -n "Getting m2m manage-realm role id : "
      m2mManageRealmRoleId=`keycloak get "/admin/realms/$realm/users/$m2mUserId/role-mappings/clients/$realmManagementId/available"  | jq -r '.[] | select(.name=="manage-realm") | .id'`; rc=$?
      echo $m2mManageRealmRoleId
      [[ $rc == 0 ]] || return $rc

      echo "Adding managerealm role to m2m user"
      keycloak post "/admin/realms/$realm/users/$m2mUserId/role-mappings/clients/$realmManagementId" "[{\"id\":\"$m2mManageRealmRoleId\",\"name\":\"manage-realm\"}]" || return $?

      addUserToGroup $m2mUsername $adminGroup || return $?
      addUserToGroup $m2mUsername $superAdminGroup || return $?

      if [[ "$provider" == "sso" ]]; then
        createSSO
      fi
    else
      echo "Provider is $provider. Ignoring  creating m2m user & groups inside keycloak"
    fi
    echo -e '\nKeycloak update DONE !'
}

function createSSO() {
    local rc=0

    echo 'Creating exchange-cryptr client'
    keycloak post "/admin/realms/$realm/clients" '{
      "enabled":true,
      "attributes":{},
      "redirectUris":[],
      "clientId":"exchange-cryptr",
      "protocol":"openid-connect"
    }' || return $?

    echo 'Getting exchange-cryptr client ID : '
    exchangeCryptrClientId=`keycloak get "/admin/realms/$realm/clients" | jq -r '.[] | select(.clientId=="exchange-cryptr") | .id'` || {
      echo "Unable to get exchange-cryptr client id"
      return 1
    } >&2

    echo 'Update exchange-cryptr client : '
    keycloak put "/admin/realms/$realm/clients/$exchangeCryptrClientId" '{
      "id": "'$exchangeCryptrClientId'",
      "clientId": "exchange-cryptr",
      "name": "exchange-cryptr",
      "description": "exchange-cryptr",
      "surrogateAuthRequired": false,
      "enabled": true,
      "clientAuthenticatorType": "client-secret",
      "redirectUris": [],
      "webOrigins": [],
      "notBefore": 0,
      "bearerOnly": false,
      "consentRequired": false,
      "standardFlowEnabled": false,
      "implicitFlowEnabled": false,
      "directAccessGrantsEnabled": true,
      "serviceAccountsEnabled": false,
      "publicClient": true,
      "frontchannelLogout": false,
      "protocol": "openid-connect",
      "attributes": {
        "saml.assertion.signature": "false",
        "saml.force.post.binding": "false",
        "saml.multivalued.roles": "false",
        "saml.encrypt": "false",
        "saml.server.signature": "false",
        "saml.server.signature.keyinfo.ext": "false",
        "exclude.session.state.from.auth.response": "false",
        "saml_force_name_id_format": "false",
        "saml.client.signature": "false",
        "tls.client.certificate.bound.access.tokens": "false",
        "saml.authnstatement": "false",
        "display.on.consent.screen": "false",
        "saml.onetimeuse.condition": "false"
      },
      "authenticationFlowBindingOverrides": {},
      "fullScopeAllowed": true,
      "nodeReRegistrationTimeout": -1,
      "defaultClientScopes": [
        "web-origins",
        "role_list",
        "profile",
        "roles",
        "email"
      ],
      "optionalClientScopes": [
        "address",
        "phone",
        "offline_access",
        "microprofile-jwt"
      ],
      "access": {
        "view": true,
        "configure": true,
        "manage": true
      }
    }'

    echo 'Update exchange-cryptr client, enable permission status : '
    keycloak put "/admin/realms/$realm/clients/$exchangeCryptrClientId/management/permissions" '{"enabled":true}'

    echo "Retrieving realm management id : "
    realmManagementId=`keycloak get "/admin/realms/$realm/clients" | jq -r '.[] | select(.clientId=="realm-management") | .id'`; rc=$?
    echo $realmManagementId
    [[ $rc == 0 ]] || return $rc

    echo 'Update realm management client, create exchange__acme-saagie-team : '
    response=`keycloak post "/admin/realms/$realm/clients/$realmManagementId/authz/resource-server/policy/client" '{"type":"client","logic":"POSITIVE","decisionStrategy":"UNANIMOUS","name":"exchange__acme-saagie-team","clients":["'$exchangeCryptrClientId'"]}'`
    policyId=$(echo "$response" | jq -r '.id')
    echo "policyId : $policyId"

    echo 'Create new identity provider : '
    keycloak post "/admin/realms/$realm/identity-provider/instances" '{
      "config": {
        "useJwksUrl":"true",
        "hideOnLoginPage":"",
        "loginHint":"",
        "uiLocales":"true",
        "backchannelSupported":"",
        "disableUserInfo":"",
        "validateSignature":"",
        "authorizationUrl":"'$ssoAuthorizationUrl'",
        "tokenUrl":"'$ssoTokenUrl'",
        "clientId":"'$ssoClientId'",
        "userInfoUrl":"'$ssoUserInfoUrl'",
        "clientSecret":"'$ssoClientSecret'"
      },
      "alias":"'$ssoAlias'",
      "providerId":"oidc",
      "enabled":true,
      "authenticateByDefault":false,
      "firstBrokerLoginFlowAlias":"first broker login",
      "postBrokerLoginFlowAlias":"",
      "storeToken":"",
      "addReadTokenRoleOnCreate":"",
      "trustEmail":"",
      "linkOnly":"",
      "displayName":"'$ssoAlias'"
    }'

    echo 'Get identity provider id'
    response=`keycloak get "/admin/realms/$realm/identity-provider/instances/$ssoAlias"`
    identityProviderId=$(echo "$response" | jq -r '.internalId')
    echo "Get identity provider - identityProviderId : $identityProviderId"

    echo 'Update identity provider permission enabled true: '
    response=`keycloak put "/admin/realms/$realm/identity-provider/instances/$ssoAlias/management/permissions" '{"enabled":true}'`
    tokenExchangeId=$(echo "$response" | jq -r '.scopePermissions."token-exchange"')
    echo "tokenExchangeId : $tokenExchangeId"
    resourceId=$(echo "$response" | jq -r '.resource')
    echo "resourceId : resourceId"

    echo 'Get scope id:'
    response=`keycloak get "/admin/realms/saagie/clients/$realmManagementId/authz/resource-server/resource/$resourceId/scopes"`
    scopeId=$(echo "$response" | jq -r '.[] | select(.name == "token-exchange") | .id')

    echo "scopeId : $scopeId"

    keycloak put "/admin/realms/saagie/clients/$realmManagementId/authz/resource-server/permission/scope/$tokenExchangeId" '{
      "id":"'tokenExchangeId'",
      "name":"token-exchange.permission.idp.'$identityProviderId'",
      "type":"scope",
      "logic":"POSITIVE",
      "decisionStrategy":"UNANIMOUS",
      "resources":["'$resourceId'"],
      "scopes":["'$scopeId'"],
      "policies":["'$policyId'"]
    }'

    echo -e '\nKeycloak SSO configuration DONE !'
}

function createUser() {
    login=$1
    password=$2
    echo -n "  - Creating user $login: "
    statusLogin=$(curl --write-out %{http_code} -s --output /dev/null --request GET --url "$authUrl/users/$login" --header "Saagie-Realm: $realm" --header "$authAuthHeader") || return $?
    if [[ "$statusLogin" == "404" ]]; then # Auth GET api retrieve the user from mongo AND from keycloak. If it exists only in Keycloak, this will still result in a 404
        status_create=`curl --write-out %{http_code} -s --output /dev/null --request POST --url "$authUrl/users" --header "Saagie-Realm: $realm" --header "$authAuthHeader" --header "Content-Type: application/json" --data '{"login":"'$login'", "password":"'$password'"}'` || return $?
        if [[ "$status_create" == "409" ]]; then # On insertion, there is a 409 if the user exists at least in one datasource (Keycloak OR Mongo)
          echo "[409 on create - Already exists. Nothing done.]"
        elif  [[ "$status_create" == "201" ]]; then
          echo '[OK]'
        else
          echo "ERROR while creating user $login, status $status_create"
          exit 1
        fi
    elif [[ "$statusLogin" == "200" ]]; then
        echo "[Already exists]"
    else
      echo "ERROR on user $login get user return $statusLogin"
      exit 1
    fi
}

function addUserToGroup() {
    login=$1
    group=$2
    echo -n "  - Adding user $login to group $group : "
    groupId=`keycloak get "/admin/realms/$realm/group-by-path/$group" | jq -r '.id'` &&
    userId=`keycloak get "/admin/realms/$realm/users?username=$login" | jq -r '.[] | select(.username == "'$login'") | .id '` ||
    return $?

    if [ -z "$groupId" ] ; then
      echo "Warning group $group does not exist"
    fi
    if [ -z "$userId" ] ; then
      echo "Warning user $login does not exist"
    fi

    if [ -z "$groupId" ] || [ -z "$userId" ] ; then
      echo "Warning cannot put user $1 into $2"
    else
      keycloak put "/admin/realms/$realm/users/$userId/groups/$groupId" || return $?
      echo "[OK]"
    fi
}
set -x
echo 'Getting keycloak auth token'
keycloakToken=`curl -s --request POST \
  --url $keycloakUrl/realms/master/protocol/openid-connect/token \
  --header 'content-type: application/x-www-form-urlencoded' \
  --data "client_id=admin-cli&username=admin_keycloak&password=$keycloakAdminPassword&grant_type=password" | jq -r '.access_token'` || exit $?

keycloakAuthHeader="authorization: Bearer $keycloakToken"

echo "Checking $realm realm existence"
statusRealm=$(curl --write-out %{http_code} --silent --output /dev/null --request GET --url "$keycloakUrl/admin/realms/$realm" --header "$keycloakAuthHeader") || exit $?

if [[ "$statusRealm" == "200" ]]
then
    echo "Realm $realm already exists"
    if [[ "$RECREATE_KEYCLOAK_REALM" == "true" ]]
    then
        echo "Deleting potentially existing $realm realm"
        keycloak delete "/admin/realms/$realm" &&
        createRealm ||
        exit $?
    else
       getBackendClientId || exit $?
       echo 'Realm will not be recreated. If you want to recreate it please set forceCreate=true in the customer configuration file or in helm command line'
       echo 'Stopping keycloak initialization here'

      if [ "$provider" == "freeipa" ]; then
         updateFirstNameAttributeMapper || exit $?
      else
         echo "Provider is $provider. Ignoring updateFirstNameAttributeMapper."
      fi
    fi
else
    createRealm || exit $?
fi

if [[ "$provider" == "freeipa" ]] || [[ "$provider" == "keycloak" ]] || [[ "$provider" == "sso" ]]; then
    echo 'Getting auth auth token'
    authToken=''
    dateLimit="$(date +%s)"
    dateLimit=$((dateLimit + 120)) # 120 seconds timeout
    authTokenAttempt=1
    while [[ -z "${authToken}" && "$(date +%s)" < "${dateLimit}" ]]; do
      authToken=$(curl -s -X POST --fail ${authenticationUrl}/open/authenticate -H 'Content-Type: application/json' -H "Saagie-Realm: $realm" -d '{"login":"'$m2mUsername'","password":"'$m2mPassword'"}') || {
        echo "Authentication failed. Retry... (#${authTokenAttempt})"
        sleep 10
      } >&2
      authTokenAttempt=$((authTokenAttempt + 1))
    done
    [[ ! -z "${authToken}" ]] || {
      echo "Max attempt reached. Aborting..."
      exit 1
    } >&2
    echo 'Authentication done'

    authAuthHeader="authorization: Bearer $authToken"

    # At this time auth should have taken the new configuration file for the customer
    # and be able to retrieve groups from ipa.
    getUserGroups

    echo "Adding customer user ($customerUserLogin)" &&
    createUser $customerUserLogin $customerPassword  &&
    addUserToGroup $customerUserLogin $adminGroup    ||
    exit $?
fi

echo 'Done!'
