local DROP_RECORD               = -1
local KEEP_RECORD_AND_TIMESTAMP = 0
local NEW_RECORD_AND_TIMESTAMP  = 1
local NEW_RECORD_KEEP_TIMESTAMP = 2 -- The code 2 is supported from FluentBit v1.4.3.
local COMMON_NAMESPACE = os.getenv("COMMON_NAMESPACE")

function log(level, message, data)
  local meta = ""
  if data ~= nil then
    for k, v in pairs(data) do
      meta = meta .. " " .. k .. "=" .. v
    end
  end

  io.stderr:write("[" .. os.date("%Y-%m-%d %X") .. "] [" .. string.format("%5s", level) .. "] [saagie] " .. message .. meta .."\n")
end

function string.starts(String,Start)
   return string.sub(String,1,string.len(Start))==Start
end

function filter_on_installation_id(tag, timestamp, record)
  if record["kubernetes"] ~= nil
    and record["kubernetes"]["labels"] ~= nil
    and record["kubernetes"]["labels"]["io.saagie/installationId"] == COMMON_NAMESPACE
    and record["kubernetes"]["container_name"] ~= nil then

    record["container_name"] = record["kubernetes"]["container_name"]
    return NEW_RECORD_KEEP_TIMESTAMP, timestamp, record
  end
  return DROP_RECORD -- Do not keep record
end

function keep_only_app_job_or_condition(tag, timestamp, record)
  local type = record["kubernetes"]["labels"]["io.saagie/type"]
  if type == "job"
    or type == "app"
    or type == "condition" then

    record["type"] = type
    return NEW_RECORD_KEEP_TIMESTAMP, timestamp, record
  end

  log("warn", "Not a valid app, job or pipeline's condition", {
    container_name = record["kubernetes"]["container_name"]
  })
  return DROP_RECORD -- Do not keep record
end

function record_project_context(tag, timestamp, record)
  record["project_id"]  = record["kubernetes"]["labels"]["io.saagie/project-id"]
  record["customer"]    = record["kubernetes"]["labels"]["io.saagie/customer"]
  record["platform_id"] = record["kubernetes"]["labels"]["io.saagie/platform-id"]
  return NEW_RECORD_KEEP_TIMESTAMP, timestamp, record
end

-- Don't forget update fluent-bit-mongo (https://github.com/saagie/fluent-bit-mongo) if you update data records
function record_app_job_and_condition_contextual_info(tag, timestamp, record)
  if record["type"] == "job" then
    if record["kubernetes"]["labels"]["io.saagie/job-execution-id"] ~= nil then
        record["job_execution_id"] = record["kubernetes"]["labels"]["io.saagie/job-execution-id"]
    else
        record["job_execution_id"] = record["kubernetes"]["pod_id"]
    end
  elseif record["type"] == "app" then
    record["app_execution_id"] = record["kubernetes"]["labels"]["execution.app.saagie.io/id"]
    record["app_id"]           = record["kubernetes"]["labels"]["app.saagie.io/id"]
    record["container_id"]     = string.match(tag, ".*-project.*_main%-(.*).log")

  elseif record["type"] == "condition" then
    record["condition_execution_id"]             = record["kubernetes"]["pod_id"]
    record["condition_node_id"]                  = record["kubernetes"]["labels"]["io.saagie/condition-node-id"]
    record["pipeline_execution_id"]              = record["kubernetes"]["labels"]["io.saagie/pipeline-execution-id"]
  end
  return NEW_RECORD_KEEP_TIMESTAMP, timestamp, record
end

-- Records must match Mongo's documents defined in fluent-bit-mongo (https://github.com/saagie/fluent-bit-mongo) or will be ignored with an error thrown in pod's logs
function record_log_prefixes(tag, timestamp, record)
  if record["log"] == nil then
    return DROP_RECORD -- Fail fast: do not keep record
  end

  local container_name = record["container_name"]
  -- iterate over the stream parsers for this container
  for key, stream_parser in pairs(record["kubernetes"]["annotations"]) do
    local prefix = "logging.saagie.io/stream-parser-" .. container_name
    if string.starts(key, prefix) then
      -- if the log matches the stream parser then flag it and get done with it
      local matches_parser = string.find(record["log"], stream_parser) ~= nil
      local log_prefix = string.sub(key, string.len(prefix)+2)
      if matches_parser then
        if string.starts(record["log"], "[" .. log_prefix .. "]") then
          record["log_prefix"] = log_prefix
        else
          record["stream"] = log_prefix .. "_" .. record["stream"]
        end
        return NEW_RECORD_KEEP_TIMESTAMP, timestamp, record
      end
    end
  end

  -- we reach here if
  -- - there are no stream parsers for this container // which means all its logs are considered unprefixed/vanilla
  -- - there are stream parsers for this container but none matched the log line // which means this log is considered unprefixed/vanilla

  -- keep record if the config does not say we should exclude unprefixed/vanilla logs for this container
  if record["kubernetes"]["annotations"]["logging.saagie.io/exclude-vanilla-" .. container_name] == "true" then
    return DROP_RECORD -- Do not keep record
  end

  return KEEP_RECORD_AND_TIMESTAMP, timestamp, record
end
